@@ -2,7 +2,7 @@
  *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2015 Artem Pavlenko
+ * Copyright (C) 2025 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,7 +33,7 @@
 #include <mapnik/query.hpp>
 #include <mapnik/datasource.hpp>
 #include <mapnik/feature_type_style.hpp>
-#include <mapnik/box2d.hpp>
+#include <mapnik/geometry/box2d.hpp>
 #include <mapnik/layer.hpp>
 #include <mapnik/rule.hpp>
 #include <mapnik/rule_cache.hpp>
@@ -41,7 +41,7 @@
 #include <mapnik/expression_evaluator.hpp>
 #include <mapnik/scale_denominator.hpp>
 #include <mapnik/projection.hpp>
-#include <mapnik/proj_transform.hpp>
+#include <mapnik/proj_transform_cache.hpp>
 #include <mapnik/util/featureset_buffer.hpp>
 #include <mapnik/util/variant.hpp>
 #include <mapnik/symbolizer_dispatch.hpp>
@@ -50,8 +50,7 @@
 #include <vector>
 #include <stdexcept>
 
-namespace mapnik
-{
+namespace mapnik {
 
 // Store material for layer rendering in a two step process
 struct layer_rendering_material
@@ -63,17 +62,18 @@ struct layer_rendering_material
     std::vector<feature_type_style const*> active_styles_;
     std::vector<featureset_ptr> featureset_ptr_list_;
     std::vector<rule_cache> rule_caches_;
+    std::vector<layer_rendering_material> materials_;
 
     layer_rendering_material(layer const& lay, projection const& dest)
-        :
-        lay_(lay),
-        proj0_(dest),
-        proj1_(lay.srs(),true) {}
+        : lay_(lay)
+        , proj0_(dest)
+        , proj1_(lay.srs(), true)
+    {}
 
-    layer_rendering_material(layer_rendering_material && rhs) = default;
+    layer_rendering_material(layer_rendering_material&& rhs) = default;
 };
 
-template <typename Processor>
+template<typename Processor>
 feature_style_processor<Processor>::feature_style_processor(Map const& m, double scale_factor)
     : m_(m)
 {
@@ -84,35 +84,19 @@ feature_style_processor<Processor>::feature_style_processor(Map const& m, double
     }
 }
 
-template <typename Processor>
-void feature_style_processor<Processor>::apply(double scale_denom)
+template<typename Processor>
+void feature_style_processor<Processor>::prepare_layers(layer_rendering_material& parent_mat,
+                                                        std::vector<layer> const& layers,
+                                                        feature_style_context_map& ctx_map,
+                                                        Processor& p,
+                                                        double scale_denom)
 {
-    Processor & p = static_cast<Processor&>(*this);
-    p.start_map_processing(m_);
-
-    projection proj(m_.srs(),true);
-    if (scale_denom <= 0.0)
-        scale_denom = mapnik::scale_denominator(m_.scale(),proj.is_geographic());
-    scale_denom *= p.scale_factor(); // FIXME - we might want to comment this out
-
-    // Asynchronous query supports:
-    // This is a two steps process,
-    // first we setup all queries at layer level
-    // in a second time, we fetch the results and
-    // do the actual rendering
-
-    std::vector<layer_rendering_material> mat_list;
-
-    // Define processing context map used by datasources
-    // implementing asynchronous queries
-    feature_style_context_map ctx_map;
-
-    for ( layer const& lyr : m_.layers() )
+    for (layer const& lyr : layers)
     {
         if (lyr.visible(scale_denom))
         {
             std::set<std::string> names;
-            layer_rendering_material mat(lyr, proj);
+            layer_rendering_material mat(lyr, parent_mat.proj0_);
 
             prepare_layer(mat,
                           ctx_map,
@@ -128,32 +112,55 @@ void feature_style_processor<Processor>::apply(double scale_denom)
             // Store active material
             if (!mat.active_styles_.empty())
             {
-                mat_list.emplace_back(std::move(mat));
+                prepare_layers(mat, lyr.layers(), ctx_map, p, scale_denom);
+                parent_mat.materials_.emplace_back(std::move(mat));
             }
         }
     }
+}
 
-    for ( layer_rendering_material const & mat : mat_list )
+template<typename Processor>
+void feature_style_processor<Processor>::apply(double scale_denom)
+{
+    Processor& p = static_cast<Processor&>(*this);
+    p.start_map_processing(m_);
+
+    projection proj(m_.srs(), true);
+    if (scale_denom <= 0.0)
+        scale_denom = mapnik::scale_denominator(m_.scale(), proj.is_geographic());
+    scale_denom *= p.scale_factor(); // FIXME - we might want to comment this out
+
+    // Asynchronous query supports:
+    // This is a two steps process,
+    // first we setup all queries at layer level
+    // in a second time, we fetch the results and
+    // do the actual rendering
+
+    // Define processing context map used by datasources
+    // implementing asynchronous queries
+    feature_style_context_map ctx_map;
+
+    if (!m_.layers().empty())
     {
-        if (!mat.active_styles_.empty())
-        {
-            render_material(mat, p);
-        }
+        layer_rendering_material root_mat(m_.layers().front(), proj);
+        prepare_layers(root_mat, m_.layers(), ctx_map, p, scale_denom);
+
+        render_submaterials(root_mat, p);
     }
 
     p.end_map_processing(m_);
 }
 
-template <typename Processor>
+template<typename Processor>
 void feature_style_processor<Processor>::apply(mapnik::layer const& lyr,
                                                std::set<std::string>& names,
                                                double scale_denom)
 {
-    Processor & p = static_cast<Processor&>(*this);
+    Processor& p = static_cast<Processor&>(*this);
     p.start_map_processing(m_);
-    projection proj(m_.srs(),true);
+    projection proj(m_.srs(), true);
     if (scale_denom <= 0.0)
-        scale_denom = mapnik::scale_denominator(m_.scale(),proj.is_geographic());
+        scale_denom = mapnik::scale_denominator(m_.scale(), proj.is_geographic());
     scale_denom *= p.scale_factor();
 
     if (lyr.visible(scale_denom))
@@ -175,9 +182,9 @@ void feature_style_processor<Processor>::apply(mapnik::layer const& lyr,
 /*!
  * \brief render a layer given a projection and scale.
  */
-template <typename Processor>
+template<typename Processor>
 void feature_style_processor<Processor>::apply_to_layer(layer const& lay,
-                                                        Processor & p,
+                                                        Processor& p,
                                                         projection const& proj0,
                                                         double scale,
                                                         double scale_denom,
@@ -188,29 +195,27 @@ void feature_style_processor<Processor>::apply_to_layer(layer const& lay,
                                                         std::set<std::string>& names)
 {
     feature_style_context_map ctx_map;
-    layer_rendering_material  mat(lay, proj0);
+    layer_rendering_material mat(lay, proj0);
 
-    prepare_layer(mat,
-                  ctx_map,
-                  p,
-                  scale,
-                  scale_denom,
-                  width,
-                  height,
-                  extent,
-                  buffer_size,
-                  names);
+    prepare_layer(mat, ctx_map, p, scale, scale_denom, width, height, extent, buffer_size, names);
+
+    prepare_layers(mat, lay.layers(), ctx_map, p, scale_denom);
 
     if (!mat.active_styles_.empty())
     {
-        render_material(mat,p);
+        p.start_layer_processing(mat.lay_, mat.layer_ext2_);
+
+        render_material(mat, p);
+        render_submaterials(mat, p);
+
+        p.end_layer_processing(mat.lay_);
     }
 }
 
-template <typename Processor>
-void feature_style_processor<Processor>::prepare_layer(layer_rendering_material & mat,
-                                                       feature_style_context_map & ctx_map,
-                                                       Processor & p,
+template<typename Processor>
+void feature_style_processor<Processor>::prepare_layer(layer_rendering_material& mat,
+                                                       feature_style_context_map& ctx_map,
+                                                       Processor& p,
                                                        double scale,
                                                        double scale_denom,
                                                        unsigned width,
@@ -226,27 +231,24 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
     std::size_t num_styles = style_names.size();
     if (num_styles == 0)
     {
-        MAPNIK_LOG_DEBUG(feature_style_processor)
-            << "feature_style_processor: No style for layer=" << lay.name();
+        MAPNIK_LOG_DEBUG(feature_style_processor) << "feature_style_processor: No style for layer=" << lay.name();
         return;
     }
 
     mapnik::datasource_ptr ds = lay.datasource();
     if (!ds)
     {
-        MAPNIK_LOG_DEBUG(feature_style_processor)
-            << "feature_style_processor: No datasource for layer=" << lay.name();
+        MAPNIK_LOG_DEBUG(feature_style_processor) << "feature_style_processor: No datasource for layer=" << lay.name();
         return;
     }
 
     processor_context_ptr current_ctx = ds->get_context(ctx_map);
-    proj_transform prj_trans(mat.proj0_,mat.proj1_);
+    proj_transform const* proj_trans_ptr = proj_transform_cache::get(mat.proj0_.params(), mat.proj1_.params());
+    box2d<double> query_ext = extent;            // unbuffered
+    box2d<double> buffered_query_ext(query_ext); // buffered
 
-    box2d<double> query_ext = extent; // unbuffered
-    box2d<double> buffered_query_ext(query_ext);  // buffered
-
-    double buffer_padding = 2.0 * scale;
-    boost::optional<int> layer_buffer_size = lay.buffer_size();
+    double buffer_padding = 2.0 * scale * p.scale_factor();
+    const auto layer_buffer_size = lay.buffer_size();
     if (layer_buffer_size) // if layer overrides buffer size, use this value to compute buffered extent
     {
         buffer_padding *= *layer_buffer_size;
@@ -259,38 +261,39 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
     buffered_query_ext.height(query_ext.height() + buffer_padding);
 
     // clip buffered extent by maximum extent, if supplied
-    boost::optional<box2d<double> > const& maximum_extent = m_.maximum_extent();
+    auto&& maximum_extent = m_.maximum_extent();
     if (maximum_extent)
     {
         buffered_query_ext.clip(*maximum_extent);
     }
 
     box2d<double> layer_ext = lay.envelope();
+    const box2d<double> buffered_query_ext_map_srs = buffered_query_ext;
     bool fw_success = false;
     bool early_return = false;
 
     // first, try intersection of map extent forward projected into layer srs
-    if (prj_trans.forward(buffered_query_ext, PROJ_ENVELOPE_POINTS) && buffered_query_ext.intersects(layer_ext))
+    if (proj_trans_ptr->forward(buffered_query_ext, PROJ_ENVELOPE_POINTS) && buffered_query_ext.intersects(layer_ext))
     {
         fw_success = true;
         layer_ext.clip(buffered_query_ext);
     }
     // if no intersection and projections are also equal, early return
-    else if (prj_trans.equal())
+    else if (proj_trans_ptr->equal())
     {
         early_return = true;
     }
     // next try intersection of layer extent back projected into map srs
-    else if (prj_trans.backward(layer_ext, PROJ_ENVELOPE_POINTS) && buffered_query_ext.intersects(layer_ext))
+    else if (proj_trans_ptr->backward(layer_ext, PROJ_ENVELOPE_POINTS) &&
+             buffered_query_ext_map_srs.intersects(layer_ext))
     {
-        layer_ext.clip(buffered_query_ext);
+        layer_ext.clip(buffered_query_ext_map_srs);
         // forward project layer extent back into native projection
-        if (! prj_trans.forward(layer_ext, PROJ_ENVELOPE_POINTS))
+        if (!proj_trans_ptr->forward(layer_ext, PROJ_ENVELOPE_POINTS))
         {
             MAPNIK_LOG_ERROR(feature_style_processor)
-                << "feature_style_processor: Layer=" << lay.name()
-                << " extent=" << layer_ext << " in map projection "
-                << " did not reproject properly back to layer projection";
+              << "feature_style_processor: Layer=" << lay.name() << " extent=" << layer_ext << " in map projection "
+              << " did not reproject properly back to layer projection";
         }
     }
     else
@@ -299,7 +302,7 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
         early_return = true;
     }
 
-    std::vector<feature_type_style const*> & active_styles = mat.active_styles_;
+    std::vector<feature_type_style const*>& active_styles = mat.active_styles_;
 
     if (early_return)
     {
@@ -307,7 +310,7 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
         // https://github.com/mapnik/mapnik/issues/1477
         for (std::string const& style_name : style_names)
         {
-            boost::optional<feature_type_style const&> style=m_.find_style(style_name);
+            boost::optional<feature_type_style const&> style = m_.find_style(style_name);
             if (!style)
             {
                 continue;
@@ -332,45 +335,44 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
         query_ext.clip(*maximum_extent);
     }
 
-    box2d<double> & layer_ext2 = mat.layer_ext2_;
+    box2d<double>& layer_ext2 = mat.layer_ext2_;
 
     layer_ext2 = lay.envelope();
     if (fw_success)
     {
-        if (prj_trans.forward(query_ext, PROJ_ENVELOPE_POINTS))
+        if (proj_trans_ptr->forward(query_ext, PROJ_ENVELOPE_POINTS))
         {
             layer_ext2.clip(query_ext);
         }
     }
     else
     {
-        if (prj_trans.backward(layer_ext2, PROJ_ENVELOPE_POINTS))
+        if (proj_trans_ptr->backward(layer_ext2, PROJ_ENVELOPE_POINTS))
         {
             layer_ext2.clip(query_ext);
-            prj_trans.forward(layer_ext2, PROJ_ENVELOPE_POINTS);
+            proj_trans_ptr->forward(layer_ext2, PROJ_ENVELOPE_POINTS);
         }
     }
 
-    std::vector<rule_cache> & rule_caches = mat.rule_caches_;
+    std::vector<rule_cache>& rule_caches = mat.rule_caches_;
     attribute_collector collector(names);
 
     // iterate through all named styles collecting active styles and attribute names
     for (std::string const& style_name : style_names)
     {
-        boost::optional<feature_type_style const&> style=m_.find_style(style_name);
+        boost::optional<feature_type_style const&> style = m_.find_style(style_name);
         if (!style)
         {
-            MAPNIK_LOG_ERROR(feature_style_processor)
-                << "feature_style_processor: Style=" << style_name
-                << " required for layer=" << lay.name() << " does not exist.";
+            MAPNIK_LOG_ERROR(feature_style_processor) << "feature_style_processor: Style=" << style_name
+                                                      << " required for layer=" << lay.name() << " does not exist.";
 
             continue;
         }
 
-        std::vector<rule> const& rules = style->get_rules();
+        std::vector<rule> const& style_rules = style->get_rules();
         bool active_rules = false;
         rule_cache rc;
-        for(rule const& r : rules)
+        for (rule const& r : style_rules)
         {
             if (r.active(scale_denom))
             {
@@ -392,12 +394,11 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
         return;
     }
 
-    double qw = query_ext.width()>0 ? query_ext.width() : 1;
-    double qh = query_ext.height()>0 ? query_ext.height() : 1;
-    query::resolution_type res(width/qw,
-                               height/qh);
+    double qw = query_ext.width() > 0 ? query_ext.width() : 1;
+    double qh = query_ext.height() > 0 ? query_ext.height() : 1;
+    query::resolution_type res(width / qw, height / qh);
 
-    query q(layer_ext,res,scale_denom,extent);
+    query q(layer_ext, res, scale_denom, extent);
     q.set_variables(p.variables());
 
     if (p.attribute_collection_policy() == COLLECT_ALL)
@@ -426,27 +427,42 @@ void feature_style_processor<Processor>::prepare_layer(layer_rendering_material
 
     bool cache_features = lay.cache_features() && active_styles.size() > 1;
 
-    std::vector<featureset_ptr> & featureset_ptr_list = mat.featureset_ptr_list_;
+    std::vector<featureset_ptr>& featureset_ptr_list = mat.featureset_ptr_list_;
     if (!group_by.empty() || cache_features)
     {
-        featureset_ptr_list.push_back(ds->features_with_context(q,current_ctx));
+        featureset_ptr_list.push_back(ds->features_with_context(q, current_ctx));
     }
     else
     {
-        for(std::size_t i = 0; i < active_styles.size(); ++i)
+        for (std::size_t i = 0; i < active_styles.size(); ++i)
         {
-            featureset_ptr_list.push_back(ds->features_with_context(q,current_ctx));
+            featureset_ptr_list.push_back(ds->features_with_context(q, current_ctx));
         }
     }
 }
 
-
-template <typename Processor>
-void feature_style_processor<Processor>::render_material(layer_rendering_material const & mat,
-                                                         Processor & p )
+template<typename Processor>
+void feature_style_processor<Processor>::render_submaterials(layer_rendering_material const& parent_mat, Processor& p)
 {
-    std::vector<feature_type_style const*> const & active_styles = mat.active_styles_;
-    std::vector<featureset_ptr> const & featureset_ptr_list = mat.featureset_ptr_list_;
+    for (layer_rendering_material const& mat : parent_mat.materials_)
+    {
+        if (!mat.active_styles_.empty())
+        {
+            p.start_layer_processing(mat.lay_, mat.layer_ext2_);
+
+            render_material(mat, p);
+            render_submaterials(mat, p);
+
+            p.end_layer_processing(mat.lay_);
+        }
+    }
+}
+
+template<typename Processor>
+void feature_style_processor<Processor>::render_material(layer_rendering_material const& mat, Processor& p)
+{
+    std::vector<feature_type_style const*> const& active_styles = mat.active_styles_;
+    std::vector<featureset_ptr> const& featureset_ptr_list = mat.featureset_ptr_list_;
     if (featureset_ptr_list.empty())
     {
         // The datasource wasn't queried because of early return
@@ -459,14 +475,10 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
         return;
     }
 
-    p.start_layer_processing(mat.lay_, mat.layer_ext2_);
-
     layer const& lay = mat.lay_;
 
-    std::vector<rule_cache> const & rule_caches = mat.rule_caches_;
-
-    proj_transform prj_trans(mat.proj0_,mat.proj1_);
-
+    std::vector<rule_cache> const& rule_caches = mat.rule_caches_;
+    proj_transform const* proj_trans_ptr = proj_transform_cache::get(mat.proj0_.params(), mat.proj1_.params());
     bool cache_features = lay.cache_features() && active_styles.size() > 1;
 
     datasource_ptr ds = lay.datasource();
@@ -491,13 +503,8 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
                     std::size_t i = 0;
                     for (feature_type_style const* style : active_styles)
                     {
-
                         cache->prepare();
-                        render_style(p, style,
-                                     rule_caches[i],
-                                     cache,
-                                     prj_trans);
-                        ++i;
+                        render_style(p, style, rule_caches[i++], cache, *proj_trans_ptr);
                     }
                     cache->clear();
                 }
@@ -509,8 +516,7 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
             for (feature_type_style const* style : active_styles)
             {
                 cache->prepare();
-                render_style(p, style, rule_caches[i], cache, prj_trans);
-                ++i;
+                render_style(p, style, rule_caches[i++], cache, *proj_trans_ptr);
             }
             cache->clear();
         }
@@ -525,7 +531,6 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
             feature_ptr feature;
             while ((feature = features->next()))
             {
-
                 cache->push(feature);
             }
         }
@@ -533,10 +538,7 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
         for (feature_type_style const* style : active_styles)
         {
             cache->prepare();
-            render_style(p, style,
-                         rule_caches[i],
-                         cache, prj_trans);
-            ++i;
+            render_style(p, style, rule_caches[i++], cache, *proj_trans_ptr);
         }
     }
     // We only have a single style and no grouping.
@@ -547,23 +549,17 @@ void feature_style_processor<Processor>::render_material(layer_rendering_materia
         for (feature_type_style const* style : active_styles)
         {
             featureset_ptr features = *featuresets++;
-            render_style(p, style,
-                         rule_caches[i],
-                         features,
-                         prj_trans);
-            ++i;
+            render_style(p, style, rule_caches[i++], features, *proj_trans_ptr);
         }
     }
-    p.end_layer_processing(mat.lay_);
 }
 
-template <typename Processor>
-void feature_style_processor<Processor>::render_style(
-    Processor & p,
-    feature_type_style const* style,
-    rule_cache const& rc,
-    featureset_ptr features,
-    proj_transform const& prj_trans)
+template<typename Processor>
+void feature_style_processor<Processor>::render_style(Processor& p,
+                                                      feature_type_style const* style,
+                                                      rule_cache const& rc,
+                                                      featureset_ptr features,
+                                                      proj_transform const& prj_trans)
 {
     p.start_style_processing(*style);
     if (!features)
@@ -578,57 +574,58 @@ void feature_style_processor<Processor>::render_style(
     {
         bool do_else = true;
         bool do_also = false;
-        for (rule const* r : rc.get_if_rules() )
+        for (rule const* r : rc.get_if_rules())
         {
             expression_ptr const& expr = r->get_filter();
-            value_type result = util::apply_visitor(evaluate<feature_impl,value_type,attributes>(*feature,vars),*expr);
+            value_type result =
+              util::apply_visitor(evaluate<feature_impl, value_type, attributes>(*feature, vars), *expr);
             if (result.to_bool())
             {
                 was_painted = true;
-                do_else=false;
-                do_also=true;
+                do_else = false;
+                do_also = true;
                 rule::symbolizers const& symbols = r->get_symbolizers();
-                if(!p.process(symbols,*feature,prj_trans))
+                if (!p.process(symbols, *feature, prj_trans))
                 {
                     for (symbolizer const& sym : symbols)
                     {
-                        util::apply_visitor(symbolizer_dispatch<Processor>(p,*feature,prj_trans),sym);
+                        util::apply_visitor(symbolizer_dispatch<Processor>(p, *feature, prj_trans), sym);
                     }
                 }
-                if (style->get_filter_mode() == FILTER_FIRST)
+                if (style->get_filter_mode() == filter_mode_enum::FILTER_FIRST)
                 {
                     // Stop iterating over rules and proceed with next feature.
-                    do_also=false;
+                    do_also = false;
                     break;
                 }
             }
         }
         if (do_else)
         {
-            for( rule const* r : rc.get_else_rules() )
+            for (rule const* r : rc.get_else_rules())
             {
                 was_painted = true;
                 rule::symbolizers const& symbols = r->get_symbolizers();
-                if(!p.process(symbols,*feature,prj_trans))
+                if (!p.process(symbols, *feature, prj_trans))
                 {
                     for (symbolizer const& sym : symbols)
                     {
-                        util::apply_visitor(symbolizer_dispatch<Processor>(p,*feature,prj_trans),sym);
+                        util::apply_visitor(symbolizer_dispatch<Processor>(p, *feature, prj_trans), sym);
                     }
                 }
             }
         }
         if (do_also)
         {
-            for( rule const* r : rc.get_also_rules() )
+            for (rule const* r : rc.get_also_rules())
             {
                 was_painted = true;
                 rule::symbolizers const& symbols = r->get_symbolizers();
-                if(!p.process(symbols,*feature,prj_trans))
+                if (!p.process(symbols, *feature, prj_trans))
                 {
                     for (symbolizer const& sym : symbols)
                     {
-                        util::apply_visitor(symbolizer_dispatch<Processor>(p,*feature,prj_trans),sym);
+                        util::apply_visitor(symbolizer_dispatch<Processor>(p, *feature, prj_trans), sym);
                     }
                 }
             }
@@ -638,4 +635,4 @@ void feature_style_processor<Processor>::render_style(
     p.end_style_processing(*style);
 }
 
-}
+} // namespace mapnik
