@@ -2,7 +2,7 @@
  *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2016 Artem Pavlenko
+ * Copyright (C) 2025 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,40 +24,39 @@
 #define MAPNIK_TOPOJSON_UTILS_HPP
 
 // mapnik
-#include <mapnik/box2d.hpp>
+#include <mapnik/geometry/box2d.hpp>
 #include <mapnik/unicode.hpp>
 #include <mapnik/json/topology.hpp>
 #include <mapnik/json/attribute_value_visitor.hpp>
 #include <mapnik/feature_factory.hpp>
-#include <mapnik/geometry_adapters.hpp>
-#include <mapnik/geometry_correct.hpp>
+#include <mapnik/geometry/boost_adapters.hpp>
+#include <mapnik/geometry/correct.hpp>
 
-namespace mapnik { namespace topojson {
+namespace mapnik {
+namespace topojson {
 
 struct bounding_box_visitor
 {
     bounding_box_visitor(topology const& topo)
-        : topo_(topo),
-          num_arcs_(topo_.arcs.size()) {}
+        : topo_(topo)
+        , num_arcs_(topo_.arcs.size())
+    {}
 
-    box2d<double> operator() (mapnik::topojson::empty const&) const
-    {
-        return box2d<double>();
-    }
+    box2d<double> operator()(mapnik::topojson::empty const&) const { return box2d<double>(); }
 
-    box2d<double> operator() (mapnik::topojson::point const& pt) const
+    box2d<double> operator()(mapnik::topojson::point const& pt) const
     {
         double x = pt.coord.x;
         double y = pt.coord.y;
         if (topo_.tr)
         {
-            x =  x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-            y =  y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+            x = x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+            y = y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
         }
         return box2d<double>(x, y, x, y);
     }
 
-    box2d<double> operator() (mapnik::topojson::multi_point const& multi_pt) const
+    box2d<double> operator()(mapnik::topojson::multi_point const& multi_pt) const
     {
         box2d<double> bbox;
         bool first = true;
@@ -74,17 +73,17 @@ struct bounding_box_visitor
             if (first)
             {
                 first = false;
-                bbox.init(x,y,x,y);
+                bbox.init(x, y, x, y);
             }
             else
             {
-                bbox.expand_to_include(x,y);
+                bbox.expand_to_include(x, y);
             }
         }
         return bbox;
     }
 
-    box2d<double> operator() (mapnik::topojson::linestring const& line) const
+    box2d<double> operator()(mapnik::topojson::linestring const& line) const
     {
         box2d<double> bbox;
         bool first = true;
@@ -95,7 +94,6 @@ struct bounding_box_visitor
                 index_type arc_index = index < 0 ? std::abs(index) - 1 : index;
                 if (arc_index >= 0 && arc_index < static_cast<int>(num_arcs_))
                 {
-
                     double px = 0, py = 0;
                     auto const& arcs = topo_.arcs[arc_index];
                     for (auto pt : arcs.coordinates)
@@ -104,8 +102,8 @@ struct bounding_box_visitor
                         double y = pt.y;
                         if (topo_.tr)
                         {
-                            x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                            y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                            x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                            y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                         }
                         if (first)
                         {
@@ -123,7 +121,7 @@ struct bounding_box_visitor
         return bbox;
     }
 
-    box2d<double> operator() (mapnik::topojson::multi_linestring const& multi_line) const
+    box2d<double> operator()(mapnik::topojson::multi_linestring const& multi_line) const
     {
         box2d<double> bbox;
         if (num_arcs_ > 0)
@@ -144,8 +142,8 @@ struct bounding_box_visitor
                             double y = pt.y;
                             if (topo_.tr)
                             {
-                                x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                                y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                                x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                                y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                             }
                             if (first)
                             {
@@ -164,7 +162,7 @@ struct bounding_box_visitor
         return bbox;
     }
 
-    box2d<double> operator() (mapnik::topojson::polygon const& poly) const
+    box2d<double> operator()(mapnik::topojson::polygon const& poly) const
     {
         box2d<double> bbox;
         if (num_arcs_ > 0)
@@ -186,14 +184,14 @@ struct bounding_box_visitor
 
                             if (topo_.tr)
                             {
-                                x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                                y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                                x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                                y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                             }
 
                             if (first)
                             {
                                 first = false;
-                                bbox.init( x, y, x, y);
+                                bbox.init(x, y, x, y);
                             }
                             else
                             {
@@ -207,7 +205,7 @@ struct bounding_box_visitor
         return bbox;
     }
 
-    box2d<double> operator() (mapnik::topojson::multi_polygon const& multi_poly) const
+    box2d<double> operator()(mapnik::topojson::multi_polygon const& multi_poly) const
     {
         box2d<double> bbox;
         if (num_arcs_ > 0)
@@ -231,14 +229,14 @@ struct bounding_box_visitor
 
                                 if (topo_.tr)
                                 {
-                                    x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                                    y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                                    x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                                    y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                                 }
 
                                 if (first)
                                 {
                                     first = false;
-                                    bbox.init( x, y, x, y);
+                                    bbox.init(x, y, x, y);
                                 }
                                 else
                                 {
@@ -252,46 +250,49 @@ struct bounding_box_visitor
         }
         return bbox;
     }
-private:
+
+  private:
     topology const& topo_;
     std::size_t num_arcs_;
 };
 
 namespace {
 
-template <typename T>
-void assign_properties(mapnik::feature_impl & feature, T const& geom, mapnik::transcoder const& tr)
+template<typename T>
+void assign_properties(mapnik::feature_impl& feature, T const& geom, mapnik::transcoder const& tr)
 {
-    if ( geom.props)
+    if (geom.props)
     {
         for (auto const& p : *geom.props)
         {
-            feature.put_new(std::get<0>(p), mapnik::util::apply_visitor(mapnik::json::attribute_value_visitor(tr),std::get<1>(p)));
+            feature.put_new(std::get<0>(p),
+                            mapnik::util::apply_visitor(mapnik::json::attribute_value_visitor(tr), std::get<1>(p)));
         }
     }
 }
 
-}
+} // namespace
 
-template <typename Context>
+template<typename Context>
 struct feature_generator
 {
-    feature_generator(Context & ctx,  mapnik::transcoder const& tr, topology const& topo, std::size_t feature_id)
-        : ctx_(ctx),
-          tr_(tr),
-          topo_(topo),
-          num_arcs_(topo.arcs.size()),
-          feature_id_(feature_id) {}
+    feature_generator(Context& ctx, mapnik::transcoder const& tr, topology const& topo, std::size_t feature_id)
+        : ctx_(ctx)
+        , tr_(tr)
+        , topo_(topo)
+        , num_arcs_(topo.arcs.size())
+        , feature_id_(feature_id)
+    {}
 
-    feature_ptr operator() (point const& pt) const
+    feature_ptr operator()(point const& pt) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         double x = pt.coord.x;
         double y = pt.coord.y;
         if (topo_.tr)
         {
-            x =  x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-            y =  y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+            x = x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+            y = y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
         }
         mapnik::geometry::point<double> point(x, y);
         feature->set_geometry(std::move(point));
@@ -299,9 +300,9 @@ struct feature_generator
         return feature;
     }
 
-    feature_ptr operator() (multi_point const& multi_pt) const
+    feature_ptr operator()(multi_point const& multi_pt) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         mapnik::geometry::multi_point<double> multi_point;
         multi_point.reserve(multi_pt.points.size());
         for (auto const& pt : multi_pt.points)
@@ -310,19 +311,19 @@ struct feature_generator
             double y = pt.y;
             if (topo_.tr)
             {
-                x =  x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                y =  y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                x = x * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                y = y * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
             }
-            multi_point.add_coord(x, y);
+            multi_point.emplace_back(x, y);
         }
         feature->set_geometry(std::move(multi_point));
         assign_properties(*feature, multi_pt, tr_);
         return feature;
     }
 
-    feature_ptr operator() (linestring const& line) const
+    feature_ptr operator()(linestring const& line) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         if (num_arcs_ > 0)
         {
             mapnik::geometry::line_string<double> line_string;
@@ -341,10 +342,10 @@ struct feature_generator
                         double y = pt.y;
                         if (topo_.tr)
                         {
-                            x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                            y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                            x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                            y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                         }
-                        line_string.add_coord(x,y);
+                        line_string.emplace_back(x, y);
                     }
                 }
             }
@@ -354,9 +355,9 @@ struct feature_generator
         return feature;
     }
 
-    feature_ptr operator() (multi_linestring const& multi_line) const
+    feature_ptr operator()(multi_linestring const& multi_line) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         if (num_arcs_ > 0)
         {
             mapnik::geometry::multi_line_string<double> multi_line_string;
@@ -380,12 +381,11 @@ struct feature_generator
                             double y = pt.y;
                             if (topo_.tr)
                             {
-                                x =  (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
-                                y =  (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
+                                x = (px += x) * (*topo_.tr).scale_x + (*topo_.tr).translate_x;
+                                y = (py += y) * (*topo_.tr).scale_y + (*topo_.tr).translate_y;
                             }
-                            line_string.add_coord(x, y);
+                            line_string.emplace_back(x, y);
                         }
-
                     }
                 }
                 multi_line_string.push_back(std::move(line_string));
@@ -399,15 +399,14 @@ struct feature_generator
         return feature;
     }
 
-    feature_ptr operator() (polygon const& poly) const
+    feature_ptr operator()(polygon const& poly) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         if (num_arcs_ > 0)
         {
             std::vector<mapnik::topojson::coordinate> processed_coords;
             mapnik::geometry::polygon<double> polygon;
-            if (poly.rings.size() > 1) polygon.interior_rings.reserve(poly.rings.size() - 1);
-            bool first = true;
+            polygon.reserve(poly.rings.size());
             bool hit = false;
             for (auto const& ring : poly.rings)
             {
@@ -424,7 +423,7 @@ struct feature_generator
                         auto const& coords = arcs.coordinates;
                         processed_coords.clear();
                         processed_coords.reserve(coords.size());
-                        for (auto const& pt : coords )
+                        for (auto const& pt : coords)
                         {
                             double x = pt.x;
                             double y = pt.y;
@@ -432,10 +431,10 @@ struct feature_generator
                             if (topo_.tr)
                             {
                                 transform const& tr = *topo_.tr;
-                                x =  (px += x) * tr.scale_x + tr.translate_x;
-                                y =  (py += y) * tr.scale_y + tr.translate_y;
+                                x = (px += x) * tr.scale_x + tr.translate_x;
+                                y = (py += y) * tr.scale_y + tr.translate_y;
                             }
-                            processed_coords.emplace_back(coordinate{x,y});
+                            processed_coords.emplace_back(coordinate{x, y});
                         }
                         linear_ring.reserve(linear_ring.size() + processed_coords.size());
                         if (reverse)
@@ -454,15 +453,7 @@ struct feature_generator
                         }
                     }
                 }
-                if (first)
-                {
-                    first = false;
-                    polygon.set_exterior_ring(std::move(linear_ring));
-                }
-                else
-                {
-                    polygon.add_hole(std::move(linear_ring));
-                }
+                polygon.push_back(std::move(linear_ring));
             }
             if (hit)
             {
@@ -474,9 +465,9 @@ struct feature_generator
         return feature;
     }
 
-    feature_ptr operator() (multi_polygon const& multi_poly) const
+    feature_ptr operator()(multi_polygon const& multi_poly) const
     {
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_,feature_id_));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx_, feature_id_));
         if (num_arcs_ > 0)
         {
             std::vector<mapnik::topojson::coordinate> processed_coords;
@@ -485,9 +476,8 @@ struct feature_generator
             bool hit = false;
             for (auto const& poly : multi_poly.polygons)
             {
-                bool first = true;
                 mapnik::geometry::polygon<double> polygon;
-                if (poly.size() > 1) polygon.interior_rings.reserve(poly.size() - 1);
+                polygon.reserve(poly.size());
 
                 for (auto const& ring : poly)
                 {
@@ -504,7 +494,7 @@ struct feature_generator
                             auto const& coords = arcs.coordinates;
                             processed_coords.clear();
                             processed_coords.reserve(coords.size());
-                            for (auto const& pt : coords )
+                            for (auto const& pt : coords)
                             {
                                 double x = pt.x;
                                 double y = pt.y;
@@ -512,10 +502,10 @@ struct feature_generator
                                 if (topo_.tr)
                                 {
                                     transform const& tr = *topo_.tr;
-                                    x =  (px += x) * tr.scale_x + tr.translate_x;
-                                    y =  (py += y) * tr.scale_y + tr.translate_y;
+                                    x = (px += x) * tr.scale_x + tr.translate_x;
+                                    y = (py += y) * tr.scale_y + tr.translate_y;
                                 }
-                                processed_coords.emplace_back(coordinate{x,y});
+                                processed_coords.emplace_back(coordinate{x, y});
                             }
 
                             using namespace boost::adaptors;
@@ -524,27 +514,19 @@ struct feature_generator
                             {
                                 for (auto const& c : (processed_coords | reversed))
                                 {
-                                    linear_ring.add_coord(c.x, c.y);
+                                    linear_ring.emplace_back(c.x, c.y);
                                 }
                             }
                             else
                             {
                                 for (auto const& c : processed_coords)
                                 {
-                                    linear_ring.add_coord(c.x, c.y);
+                                    linear_ring.emplace_back(c.x, c.y);
                                 }
                             }
                         }
                     }
-                    if (first)
-                    {
-                        first = false;
-                        polygon.set_exterior_ring(std::move(linear_ring));
-                    }
-                    else
-                    {
-                        polygon.add_hole(std::move(linear_ring));
-                    }
+                    polygon.push_back(std::move(linear_ring));
                 }
                 multi_polygon.push_back(std::move(polygon));
             }
@@ -559,19 +541,19 @@ struct feature_generator
     }
 
     template<typename T>
-    feature_ptr operator() (T const& ) const
+    feature_ptr operator()(T const&) const
     {
         return feature_ptr();
     }
 
-    Context & ctx_;
+    Context& ctx_;
     mapnik::transcoder const& tr_;
     topology const& topo_;
     std::size_t num_arcs_;
     std::size_t feature_id_;
 };
 
+} // namespace topojson
+} // namespace mapnik
 
-}}
-
-#endif //MAPNIK_TOPOJSON_UTILS_HPP
+#endif // MAPNIK_TOPOJSON_UTILS_HPP
