@@ -2,7 +2,7 @@
  *
  * This file is part of Mapnik (c++ mapping toolkit)
  *
- * Copyright (C) 2015 Artem Pavlenko
+ * Copyright (C) 2025 Artem Pavlenko
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,18 +28,18 @@
 #include <mapnik/config.hpp>
 #include <mapnik/util/noncopyable.hpp>
 
-#pragma GCC diagnostic push
+#include <mapnik/warning.hpp>
+MAPNIK_DISABLE_WARNING_PUSH
 #include <mapnik/warning_ignore_agg.hpp>
 #include "agg_basics.h"
-#pragma GCC diagnostic pop
+MAPNIK_DISABLE_WARNING_POP
 
 // stl
 #include <vector>
 #include <memory>
 #include <map>
 
-namespace mapnik
-{
+namespace mapnik {
 
 class vertex_cache;
 using vertex_cache_ptr = std::unique_ptr<vertex_cache>;
@@ -49,17 +49,26 @@ class MAPNIK_DECL vertex_cache : util::noncopyable
 {
     struct segment
     {
-        segment(double x, double y, double _length) : pos(x, y), length(_length) {}
-        pixel_position pos; //Last point of this segment, first point is implicitly defined by the previous segement in this vector
+        segment(double x, double y, double _length)
+            : pos(x, y)
+            , length(_length)
+        {}
+        pixel_position
+          pos; // Last point of this segment, first point is implicitly defined by the previous segement in this vector
         double length;
     };
 
     // The first segment always has the length 0 and just defines the starting point.
     struct segment_vector
     {
-        segment_vector() : vector(), length(0.) {}
-        void add_segment(double x, double y, double len) {
-            if (len == 0. && !vector.empty()) return; //Don't add zero length segments
+        segment_vector()
+            : vector()
+            , length(0.)
+        {}
+        void add_segment(double x, double y, double len)
+        {
+            if (len == 0. && !vector.empty())
+                return; // Don't add zero length segments
             vector.emplace_back(x, y, len);
             length += len;
         }
@@ -68,7 +77,7 @@ class MAPNIK_DECL vertex_cache : util::noncopyable
         double length;
     };
 
-public:
+  public:
     // This class has no public members to avoid acciedential modification.
     // It should only be used with save_state/restore_state.
     class state
@@ -79,40 +88,52 @@ public:
         pixel_position segment_starting_point;
         double position_;
         friend class vertex_cache;
-    public:
+
+      public:
         pixel_position const& position() const { return current_position; }
     };
 
     class scoped_state : util::noncopyable
     {
-    public:
-        scoped_state(vertex_cache &pp) : pp_(pp), state_(pp.save_state()), restored_(false) {}
-        void restore() { pp_.restore_state(state_); restored_ = true; }
-        ~scoped_state() { if (!restored_) pp_.restore_state(state_); }
+      public:
+        scoped_state(vertex_cache& pp)
+            : pp_(pp)
+            , state_(pp.save_state())
+            , restored_(false)
+        {}
+        void restore()
+        {
+            pp_.restore_state(state_);
+            restored_ = true;
+        }
+        ~scoped_state()
+        {
+            if (!restored_)
+                pp_.restore_state(state_);
+        }
         state const& get_state() const { return state_; }
-    private:
-        vertex_cache &pp_;
+
+      private:
+        vertex_cache& pp_;
         state state_;
         bool restored_;
     };
 
     ///////////////////////////////////////////////////////////////////////
 
-    template <typename T> vertex_cache(T &path);
-    vertex_cache(vertex_cache && rhs);
+    template<typename T>
+    vertex_cache(T& path);
+    vertex_cache(vertex_cache&& rhs);
 
     double length() const { return current_subpath_->length; }
 
-
     pixel_position const& current_position() const { return current_position_; }
-    double angle(double width=0.);
+    double angle(double width = 0.);
     double current_segment_angle();
     double linear_position() const { return position_; }
 
-
     // Returns a parallel line in the specified distance.
-    vertex_cache & get_offseted(double offset, double region_width);
-
+    vertex_cache& get_offseted(double offset, double region_width);
 
     // Skip a certain amount of space.
     // This functions automatically calculate new points if the position is not exactly
@@ -130,7 +151,7 @@ public:
 
     // Compatibility with standard path interface
     void rewind(unsigned);
-    unsigned vertex(double *x, double *y);
+    unsigned vertex(double* x, double* y);
 
     // State
     state save_state() const;
@@ -139,16 +160,21 @@ public:
     void reset();
 
     // position on this line closest to the target position
-    double position_closest_to(pixel_position const &target_pos);
+    double position_closest_to(pixel_position const& target_pos);
 
-private:
+  private:
     void rewind_subpath();
     bool next_segment();
     bool previous_segment();
-    void find_line_circle_intersection(
-        double cx, double cy, double radius,
-        double x1, double y1, double x2, double y2,
-        double & ix, double & iy) const;
+    void find_line_circle_intersection(double cx,
+                                       double cy,
+                                       double radius,
+                                       double x1,
+                                       double y1,
+                                       double x2,
+                                       double y2,
+                                       double& ix,
+                                       double& iy) const;
     // Position as calculated by last move/forward/next call.
     pixel_position current_position_;
     // First pixel of current segment.
@@ -179,34 +205,33 @@ private:
     double position_;
 };
 
-
-template <typename T>
-vertex_cache::vertex_cache(T & path)
-        : current_position_(),
-          segment_starting_point_(),
-          subpaths_(),
-          current_subpath_(),
-          current_segment_(),
-          vertex_segment_(),
-          vertex_subpath_(),
-          initialized_(false),
-          position_in_segment_(0.),
-          angle_(0.),
-          angle_valid_(false),
-          offseted_lines_(),
-          position_(0.)
+template<typename T>
+vertex_cache::vertex_cache(T& path)
+    : current_position_()
+    , segment_starting_point_()
+    , subpaths_()
+    , current_subpath_()
+    , current_segment_()
+    , vertex_segment_()
+    , vertex_subpath_()
+    , initialized_(false)
+    , position_in_segment_(0.)
+    , angle_(0.)
+    , angle_valid_(false)
+    , offseted_lines_()
+    , position_(0.)
 {
     path.rewind(0);
     unsigned cmd;
     double new_x = 0., new_y = 0., old_x = 0., old_y = 0.;
-    bool first = true; //current_subpath_ uninitalized
+    bool first = true; // current_subpath_ uninitalized
     while (!agg::is_stop(cmd = path.vertex(&new_x, &new_y)))
     {
         if (agg::is_move_to(cmd))
         {
-            //Create new sub path
+            // Create new sub path
             subpaths_.emplace_back();
-            current_subpath_ = subpaths_.end()-1;
+            current_subpath_ = subpaths_.end() - 1;
             current_subpath_->add_segment(new_x, new_y, 0);
             first = false;
         }
@@ -219,15 +244,15 @@ vertex_cache::vertex_cache(T & path)
             }
             double dx = old_x - new_x;
             double dy = old_y - new_y;
-            double segment_length = std::sqrt(dx*dx + dy*dy);
+            double segment_length = std::sqrt(dx * dx + dy * dy);
             current_subpath_->add_segment(new_x, new_y, segment_length);
         }
         else if (agg::is_closed(cmd) && !current_subpath_->vector.empty())
         {
-            segment const & first_segment = current_subpath_->vector[0];
+            segment const& first_segment = current_subpath_->vector[0];
             double dx = old_x - first_segment.pos.x;
             double dy = old_y - first_segment.pos.y;
-            double segment_length = std::sqrt(dx*dx + dy*dy);
+            double segment_length = std::sqrt(dx * dx + dy * dy);
             current_subpath_->add_segment(first_segment.pos.x, first_segment.pos.y, segment_length);
         }
         old_x = new_x;
@@ -235,5 +260,5 @@ vertex_cache::vertex_cache(T & path)
     }
 }
 
-}
+} // namespace mapnik
 #endif
